from typing import Optional, Dict

from PySide6.QtWidgets import QDialog, QVBoxLayout
from PySide6.QtCore import QObject

from airunner.app import App
from airunner.handlers.llm.llm_request import LLMRequest
from airunner.handlers.llm.llm_response import LLMResponse
from airunner.handlers.stablediffusion.image_request import ImageRequest
from airunner.enums import (
    EngineResponseCode,
    GeneratorSection,
    ModelStatus,
    ModelType,
    SignalCode,
    LLMActionType,
)
from airunner.setup_database import setup_database
from airunner.utils.application.create_worker import create_worker
from airunner.gui.utils.ui_dispatcher import render_ui_from_spec
from airunner.utils.application.ui_loader import (
    load_ui_file,
    load_ui_from_string,
)
from airunner.utils.audio.sound_device_manager import SoundDeviceManager


class APIServiceBase:
    def __init__(self, emit_signal):
        self.emit_signal = emit_signal


class STTAPIService(APIServiceBase):
    def audio_processor_response(self, transcription: str):
        """
        Emit a signal with the audio processor response.
        :param transcription: The response from the audio processor.
        """
        self.emit_signal(
            SignalCode.AUDIO_PROCESSOR_RESPONSE_SIGNAL,
            {"transcription": transcription},
        )


class TTSAPIService(APIServiceBase):
    def toggle(self, enabled: bool):
        """
        Emit a signal to toggle TTS on or off.
        :param enabled: True to enable TTS, False to disable.
        """
        self.emit_signal(SignalCode.TOGGLE_TTS_SIGNAL, {"enabled": enabled})

    def add_to_stream(self, response: str):
        """
        Emit a signal to add text to the TTS stream.
        :param response: The text to add to the stream.
        """
        self.emit_signal(
            SignalCode.TTS_GENERATOR_WORKER_ADD_TO_STREAM_SIGNAL,
            {"message": response},
        )

    def disable(self):
        """
        Emit a signal to disable TTS.
        """
        self.emit_signal(SignalCode.TTS_DISABLE_SIGNAL, {})


class CanvasAPIService(APIServiceBase):
    def image_from_path(self, path: str):
        """
        Emit a signal to load an image from the given path.
        :param path: The path to the image file.
        """
        self.emit_signal(
            SignalCode.CANVAS_LOAD_IMAGE_FROM_PATH_SIGNAL,
            {"image_path": path},
        )

    def clear(self):
        """
        Emit a signal to clear the canvas.
        """
        self.emit_signal(SignalCode.CANVAS_CLEAR, {})


class ARTAPIService(APIServiceBase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.canvas = CanvasAPIService(emit_signal=self.emit_signal)

    def lora_updated(self):
        """
        Emit a signal indicating that the LoRA has been updated.
        """
        self.emit_signal(SignalCode.LORA_UPDATED_SIGNAL, {})

    def embedding_updated(self):
        """
        Emit a signal indicating that the embedding has been updated.
        """
        self.emit_signal(SignalCode.EMBEDDING_UPDATED_SIGNAL, {})

    def final_progress_update(self, total: int):
        self.progress_update(total, total)

    def progress_update(self, step: int, total: int):
        """
        Emit a signal indicating the image generation progress.
        :param step: The current step in the progress.
        :param total: The total number of steps.
        """
        self.emit_signal(
            SignalCode.SD_PROGRESS_SIGNAL,
            {
                "step": step,
                "total": total,
            },
        )

    def pipeline_loaded(self, section: GeneratorSection):
        """
        Emit a signal indicating that the pipeline has been loaded.
        :param section: The section of the pipeline that has been loaded.
        """
        self.emit_signal(
            SignalCode.SD_PIPELINE_LOADED_SIGNAL,
            {"generator_section": section},
        )

    def llm_image_generated(
        self,
        prompt: str,
        second_prompt: str,
        section: GeneratorSection,
        width: int,
        height: int,
    ):
        """
        Emit a signal indicating that an image has been generated by the LLM.
        :param prompt: The prompt used for image generation.
        :param second_prompt: The second prompt used for image generation.
        :param section: The section of the pipeline that generated the image.
        :param width: The width of the generated image.
        :param height: The height of the generated image.
        """
        self.emit_signal(
            SignalCode.LLM_IMAGE_PROMPT_GENERATED_SIGNAL,
            {
                "message": {
                    "prompt": prompt,
                    "second_prompt": second_prompt,
                    "type": section,
                    "width": width,
                    "height": height,
                }
            },
        )


class ChatbotAPIService(APIServiceBase):
    def update_mood(self, mood: str):
        self.emit_signal(SignalCode.BOT_MOOD_UPDATED, {"mood": mood})


class LLMAPIService(APIServiceBase):
    def send_request(
        self,
        prompt: str,
        llm_request: Optional[LLMRequest] = None,
        action: LLMActionType = LLMActionType.CHAT,
        do_tts_reply: bool = True,
    ):
        """
        Send a request to the LLM with the given prompt and action.

        :param prompt: The prompt to send to the LLM.
        :param llm_request: Optional LLMRequest object.
        :param action: The action type for the request.
        :param do_tts_reply: Whether to do text-to-speech reply.
        :return: None
        """
        llm_request = llm_request or LLMRequest.from_default()
        llm_request.do_tts_reply = do_tts_reply

        self.emit_signal(
            SignalCode.LLM_TEXT_GENERATE_REQUEST_SIGNAL,
            {
                "llm_request": True,
                "request_data": {
                    "action": action,
                    "prompt": prompt,
                    "llm_request": llm_request,
                    "do_tts_reply": do_tts_reply,
                },
            },
        )

    def clear_history(self):
        """
        Emit a signal to clear the LLM history.
        """
        self.emit_signal(SignalCode.LLM_CLEAR_HISTORY_SIGNAL, {})


class API(App):
    def __init__(self, *args, **kwargs):
        self.llm = LLMAPIService(emit_signal=self.emit_signal)
        self.art = ARTAPIService(emit_signal=self.emit_signal)
        self.tts = TTSAPIService(emit_signal=self.emit_signal)
        self.stt = STTAPIService(emit_signal=self.emit_signal)

        # Extract the initialize_app flag and pass the rest to the parent App class
        self._initialize_app = kwargs.pop("initialize_app", True)
        initialize_gui = kwargs.pop("initialize_gui", True)
        self.signal_handlers = {
            SignalCode.SHOW_WINDOW_SIGNAL: self.show_hello_world_window,
            SignalCode.SHOW_DYNAMIC_UI_FROM_STRING_SIGNAL: self.show_dynamic_ui_from_string,
        }
        self.sounddevice_manager = SoundDeviceManager()
        super().__init__(*args, initialize_gui=initialize_gui, **kwargs)
        self._initialize_model_scanner()

    def _initialize_model_scanner(self):
        from airunner.workers.model_scanner_worker import (
            ModelScannerWorker,
        )

        if self._initialize_app:
            setup_database()
            self.model_scanner_worker = create_worker(ModelScannerWorker)
            self.model_scanner_worker.add_to_queue("scan_for_models")

    def send_llm_text_streamed_signal(self, response: LLMResponse):
        """
        Send a TTS request with the given response."

        :param response: The LLMResponse object.
        :return: None
        """
        self.emit_signal(
            SignalCode.LLM_TEXT_STREAMED_SIGNAL, {"response": response}
        )

    def send_image_request(self, image_request: Optional[ImageRequest] = None):
        """ "
        Send a request to the image generator with the given request.
        :param image_request: Optional ImageRequest object.
        :return: None
        """
        image_request = image_request or ImageRequest()
        self.emit_signal(
            SignalCode.DO_GENERATE_SIGNAL, {"image_request": image_request}
        )

    def show_hello_world_window(self):
        """
        Display a 'Hello, world!' popup window using the UI dispatcher.
        """
        spec = {
            "type": "window",
            "title": "Hello Window",
            "layout": "vertical",
            "widgets": [{"type": "label", "text": "Hello, world!"}],
        }
        dialog = QDialog(self.app.main_window)
        dialog.setWindowTitle(spec.get("title", "Untitled"))
        render_ui_from_spec(spec, dialog)
        dialog.exec()

    def show_dynamic_ui(self, ui_file_path: str):
        """
        Load and display a .ui file dynamically as a popup window.

        :param ui_file_path: Path to the .ui file.
        """
        dialog = QDialog(self.app.main_window)
        dialog.setWindowTitle("Dynamic UI")
        widget = load_ui_file(ui_file_path, dialog)
        layout = dialog.layout() or QVBoxLayout(dialog)
        layout.addWidget(widget)
        dialog.setLayout(layout)
        dialog.exec()

    def show_dynamic_ui_from_string(self, data: Dict):
        """
        Load and display a .ui file dynamically from a string as a popup window.

        :param ui_content: The content of the .ui file as a string.
        """
        ui_content = data.get("ui_content", "")

        class SignalHandler(QObject):
            def __init__(self, api: API):
                self.api = api
                super().__init__()

            def click_me_button(self):
                self.api.emit_signal(SignalCode.SHOW_WINDOW_SIGNAL)

        signal_handler = SignalHandler(api=self)
        dialog = QDialog(self.app.main_window)
        dialog.setWindowTitle("Dynamic UI with Signal")
        widget = load_ui_from_string(ui_content, dialog, signal_handler)
        layout = dialog.layout() or QVBoxLayout(dialog)
        layout.addWidget(widget)
        dialog.setLayout(layout)
        dialog.exec()

    def change_model_status(self, model: ModelType, status: ModelStatus):
        """
        Change the status of a model and emit a signal.
        :param model: The model type.
        :param status: The new status of the model.
        """
        self.emit_signal(
            SignalCode.MODEL_STATUS_CHANGED_SIGNAL,
            {"model": model, "status": status},
        )

    def worker_response(self, code: EngineResponseCode, message: Dict):
        """
        Emit a signal indicating a response from the worker.
        :param code: The response code from the worker.
        :param message: The message from the worker.
        """
        self.emit_signal(
            SignalCode.ENGINE_RESPONSE_WORKER_RESPONSE_SIGNAL,
            {"code": code, "message": message},
        )

    def quit_application(self):
        """
        Emit a signal to quit the application.
        """
        self.emit_signal(SignalCode.QUIT_APPLICATION, {})

    def application_error(self, message: str):
        """
        Emit a signal indicating an application error.
        :param message: The error message.
        """
        self.emit_signal(
            SignalCode.APPLICATION_STATUS_ERROR_SIGNAL,
            {"message": message},
        )

    def application_status(self, message: str):
        """
        Emit a signal indicating application status.
        :param message: The status message.
        """
        self.emit_signal(
            SignalCode.APPLICATION_STATUS_INFO_SIGNAL,
            {"message": message},
        )

    def update_download_log(self, message: str):
        self.emit_signal(
            SignalCode.UPDATE_DOWNLOAD_LOG,
            {"message": message},
        )

    def set_download_progress(self, current: int, total: int):
        self.emit_signal(
            SignalCode.DOWNLOAD_PROGRESS,
            {"current": current, "total": total},
        )

    def clear_download_status(self):
        self.emit_signal(SignalCode.CLEAR_DOWNLOAD_STATUS_BAR)

    def set_download_status(self, message: str):
        self.emit_signal(
            SignalCode.SET_DOWNLOAD_STATUS_LABEL,
            {"message": message},
        )

    def download_complete(self):
        self.emit_signal(SignalCode.DOWNLOAD_COMPLETE)



from sqlalchemy import Column, Integer, String, Boolean

from airunner.data.models.base import BaseModel


class ActiveGridSettings(BaseModel):
    __tablename__ = "active_grid_settings"
    id = Column(Integer, primary_key=True, autoincrement=True)
    enabled = Column(Boolean, default=True)
    render_border = Column(Boolean, default=True)
    render_fill = Column(Boolean, default=False)
    border_opacity = Column(Integer, default=50)
    fill_opacity = Column(Integer, default=50)
    border_color = Column(String, default="#00FF00")
    fill_color = Column(String, default="#FF0000")
    pos_x = Column(Integer, default=0)
    pos_y = Column(Integer, default=0)
    width = Column(Integer, default=512)
    height = Column(Integer, default=512)

    @property
    def pos(self) -> tuple[int, int]:
        x = self.pos_x if self.pos_x is not None else 0
        y = self.pos_y if self.pos_y is not None else 0
        return x, y



import unittest
from unittest.mock import MagicMock
from PySide6.QtWidgets import QWidget, QApplication

from airunner.base_plugin import BasePlugin


class TestBasePlugin(unittest.TestCase):
    """Test cases for the BasePlugin abstract base class."""

    @classmethod
    def setUpClass(cls):
        """Set up QApplication instance for the test suite."""
        if not QApplication.instance():
            cls.qapp = QApplication([])
        else:
            cls.qapp = QApplication.instance()

    @classmethod
    def tearDownClass(cls):
        """Clean up QApplication instance after tests."""
        cls.qapp.quit()

    def test_base_plugin_abstract_methods(self):
        """Test that BasePlugin enforces implementation of its abstract methods."""
        # Attempt to instantiate BasePlugin directly should fail
        with self.assertRaises(TypeError):
            BasePlugin()

    def test_base_plugin_implementation(self):
        """Test a concrete implementation of BasePlugin."""
        # Create a concrete implementation of BasePlugin
        class TestPlugin(BasePlugin):
            name = "Test Plugin"

            def get_widget(self):
                """Return a test widget."""
                return QWidget()

        # Instantiate the concrete implementation
        plugin = TestPlugin()
        
        # Verify plugin name
        self.assertEqual(plugin.name, "Test Plugin")
        
        # Verify get_widget method returns a QWidget
        widget = plugin.get_widget()
        self.assertIsInstance(widget, QWidget)

    def test_base_plugin_multiple_implementations(self):
        """Test multiple concrete implementations of BasePlugin."""
        class Plugin1(BasePlugin):
            name = "Plugin 1"
            
            def get_widget(self):
                widget = QWidget()
                widget.setObjectName("widget1")
                return widget

        class Plugin2(BasePlugin):
            name = "Plugin 2"
            
            def get_widget(self):
                widget = QWidget()
                widget.setObjectName("widget2")
                return widget

        # Create instances
        plugin1 = Plugin1()
        plugin2 = Plugin2()
        
        # Verify different names
        self.assertEqual(plugin1.name, "Plugin 1")
        self.assertEqual(plugin2.name, "Plugin 2")
        
        # Verify widgets have different object names
        widget1 = plugin1.get_widget()
        widget2 = plugin2.get_widget()
        self.assertEqual(widget1.objectName(), "widget1")
        self.assertEqual(widget2.objectName(), "widget2")


if __name__ == "__main__":
    unittest.main()


from typing import Type, Optional
from abc import ABCMeta
import os
import torch

torch.hub.set_dir(
    os.environ.get(
        "TORCH_HOME", "/home/appuser/.local/share/airunner/torch/hub"
    )
)
import librosa

from openvoice.mel_processing import spectrogram_torch
from openvoice import se_extractor
from openvoice.api import OpenVoiceBaseClass, ToneColorConverter
from melo.api import TTS

from airunner.settings import (
    AIRUNNER_BASE_PATH,
    AIRUNNER_TTS_SPEAKER_RECORDING_PATH,
    AIRUNNER_LOG_LEVEL,
)
from airunner.enums import (
    SignalCode,
    ModelType,
    ModelStatus,
    AvailableLanguage,
)
from airunner.handlers.tts.tts_model_manager import TTSModelManager
from airunner.handlers.tts.tts_request import TTSRequest
from airunner.utils.application.get_logger import get_logger


class StreamingToneColorConverter(ToneColorConverter):
    """
    Streaming implementation of ToneColorConverter.
    """

    def __init__(self, *args, **kwargs):
        OpenVoiceBaseClass.__init__(self, *args, **kwargs)
        self.version = getattr(self.hps, "_version_", "v1")
        self.logger = get_logger("AI Runner", AIRUNNER_LOG_LEVEL)

    def convert(
        self,
        audio_src_path,
        src_se,
        tgt_se,
        output_path=None,
        tau=0.3,
        message="default",
    ):
        """
        Convert audio tone color using the specified parameters.
        """
        hps = self.hps
        try:
            audio, sample_rate = librosa.load(
                audio_src_path, sr=hps.data.sampling_rate
            )
        except ValueError as e:
            print(f"Error: {e}")
            return None

        audio = torch.tensor(audio).float()

        with torch.no_grad():
            y = torch.FloatTensor(audio).to(self.device).unsqueeze(0)
            try:
                spec = spectrogram_torch(
                    y,
                    hps.data.filter_length,
                    hps.data.sampling_rate,
                    hps.data.hop_length,
                    hps.data.win_length,
                    center=False,
                ).to(self.device)
            except RuntimeError as e:
                self.logger.error(
                    f"Runtime error during spectrogram computation: {e}"
                )
                return None
            spec_lengths = torch.LongTensor([spec.size(-1)]).to(self.device)
            audio = (
                self.model.voice_conversion(
                    spec, spec_lengths, sid_src=src_se, sid_tgt=tgt_se, tau=tau
                )[0][0, 0]
                .data.cpu()
                .float()
                .numpy()
            )
            return audio


class OpenVoiceModelManager(TTSModelManager, metaclass=ABCMeta):
    """
    OpenVoice-based implementation of the TTSModelManager.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._target_se = None
        self._audio_name = None
        speaker_recording_path = os.path.expanduser(
            AIRUNNER_TTS_SPEAKER_RECORDING_PATH
        )
        self._checkpoint_converter_path: str = os.path.join(
            self.path_settings.tts_model_path,
            "openvoice/checkpoints_v2/converter",
        )
        self._output_dir: str = os.path.join(
            self.path_settings.tts_model_path, "openvoice/outputs_v2"
        )
        self._tone_color_converter: Optional[Type[ToneColorConverter]] = None
        self.model: Optional[TTS] = None
        self.src_path: str = f"{self._output_dir}/tmp.wav"
        self._speed: float = 1.0
        self._language: AvailableLanguage = AvailableLanguage.EN_NEWEST
        self._reference_speaker = os.path.expanduser(speaker_recording_path)

    @property
    def device(self):
        """
        Return the appropriate device based on CUDA availability.
        """
        use_cuda = torch.cuda.is_available()
        card_index = 0
        return f"cuda:{card_index}" if use_cuda else "cpu"

    @property
    def tone_color_converter(self) -> StreamingToneColorConverter:
        """
        Lazy-load the tone color converter.
        """
        if not self._tone_color_converter:
            self._tone_color_converter = StreamingToneColorConverter(
                f"{self._checkpoint_converter_path}/config.json",
                device=self.device,
            )
            self._tone_color_converter.load_ckpt(
                f"{self._checkpoint_converter_path}/checkpoint.pth"
            )
        return self._tone_color_converter

    def generate(self, tts_request: Type[TTSRequest]):
        """
        Generate speech using OpenVoice and apply tone color conversion.
        """
        message = tts_request.message
        speaker_ids = self.model.hps.data.spk2id
        for speaker_key in speaker_ids.keys():
            speaker_id = speaker_ids[speaker_key]
            speaker_key = speaker_key.lower().replace("_", "-")

            source_se = torch.load(
                os.path.join(
                    self.path_settings.tts_model_path,
                    f"openvoice/checkpoints_v2/base_speakers/ses/{speaker_key}.pth",
                ),
                map_location=self.device,
            )

            self.model.tts_to_file(
                message, speaker_id, self.src_path, speed=self._speed
            )

            output_path = os.path.join(
                self.path_settings.tts_model_path,
                f"openvoice/{self._output_dir}/output_v2_{speaker_key}.wav",
            )

            response = self.tone_color_converter.convert(
                audio_src_path=self.src_path,
                src_se=source_se,
                tgt_se=self._target_se,
                output_path=output_path,
            )

            if response is not None:
                self.api.tts.add_to_stream(response)

    def load(self, _target_model=None):
        """
        Load and initialize the OpenVoice model.
        """
        self.logger.debug("Initializing OpenVoice")
        self.unload()
        self.change_model_status(ModelType.TTS, ModelStatus.LOADING)
        self._initialize()
        do_download = False

        import nltk

        nltk.download("averaged_perceptron_tagger_eng")

        # if do_download:
        #     vad, vad_utils = torch.hub.load(
        #         repo_or_dir="snakers4/silero-vad",
        #         model="silero_vad",
        #         force_reload=False,
        #         onnx=False,
        #     )
        self.model = TTS(language=self._language.value, device=self.device)
        self.change_model_status(ModelType.TTS, ModelStatus.LOADED)

    def unload(self):
        """
        Unload the OpenVoice model and release resources.
        """
        self.logger.debug("Unloading OpenVoice")
        self.change_model_status(ModelType.TTS, ModelStatus.LOADING)
        self.model = None
        self.change_model_status(ModelType.TTS, ModelStatus.UNLOADED)

    def unblock_tts_generator_signal(self):
        """
        Placeholder for unblocking TTS generator signal.
        """
        pass

    def interrupt_process_signal(self):
        """
        Placeholder for interrupting the TTS process.
        """
        pass

    def _initialize(self):
        """
        Initialize OpenVoice-specific settings and resources.
        """
        try:
            os.makedirs(self._output_dir, exist_ok=True)
        except FileExistsError:
            pass

        if self._reference_speaker is None:
            self.logger.error(
                "Reference speaker is None, unable to initialize"
            )
            return

        try:
            self.logger.info(f"Loading {self._reference_speaker}")
            self._target_se, self._audio_name = se_extractor.get_se(
                audio_path=self._reference_speaker,
                vc_model=self.tone_color_converter,
                vad=True,
                target_dir=os.path.join(AIRUNNER_BASE_PATH, "processed"),
            )
        except AssertionError as e:
            torch_hub_cache_home = torch.hub.get_dir()
            self.logger.error(
                f"Failed to load from se_extractor {e} - torch_hub_cache_home={torch_hub_cache_home}"
            )
            self.api.tts.disable()

        if self._target_se is None:
            self.logger.error("Target speaker extraction returned None.")



__all__ = [
    "HuggingFaceLLM",
    "LLMModelManager",
    "LLMRequest",
    "LLMResponse",
    "LLMSettings",
    "TrainingMixin",
    "OpenrouterMistralRequest",
]


def __getattr__(name):
    if name == "HuggingFaceLLM":
        from .huggingface_llm import HuggingFaceLLM

        return HuggingFaceLLM
    elif name == "LLMModelManager":
        from .llm_model_manager import LLMModelManager

        return LLMModelManager
    elif name == "LLMRequest":
        from .llm_request import LLMRequest

        return LLMRequest
    elif name == "LLMResponse":
        from .llm_response import LLMResponse

        return LLMResponse
    elif name == "LLMSettings":
        from .llm_settings import LLMSettings

        return LLMSettings
    elif name == "TrainingMixin":
        from .training_mixin import TrainingMixin

        return TrainingMixin
    elif name == "OpenrouterMistralRequest":
        from .llm_request import OpenrouterMistralRequest

        return OpenrouterMistralRequest
    raise AttributeError(f"module {__name__} has no attribute {name}")



from sqlalchemy import Column, Integer, String, Boolean
from sqlalchemy.orm import relationship

from airunner.data.models.base import BaseModel
from airunner.data.models.image_filter_value import ImageFilterValue


class ImageFilter(BaseModel):
    __tablename__ = 'image_filter_settings'
    id = Column(Integer, primary_key=True, autoincrement=True)
    name = Column(String, nullable=False, default="")
    display_name = Column(String, nullable=False, default="")
    auto_apply = Column(Boolean, default=False)
    filter_class = Column(String, nullable=False, default="")


ImageFilter.image_filter_values = relationship(
    "ImageFilterValue", 
    order_by=ImageFilterValue.id, 
    back_populates="image_filter"
)



import sounddevice as sd
import numpy as np


def log_audio_devices():
    devices = sd.query_devices()
    print("Available audio devices:")
    for i, device in enumerate(devices):
        print(
            f"Device {i}: {device['name']}, Max Output Channels: {device['max_output_channels']}, Default Sample Rate: {device['default_samplerate']}"
        )


def play_test_tone():
    samplerate = 44100  # Standard sample rate
    duration = 2  # seconds
    frequency = 440.0  # Hz (A4)

    print(f"Playing test tone at {samplerate} Hz...")
    t = np.linspace(0, duration, int(samplerate * duration), endpoint=False)
    tone = 0.5 * np.sin(2 * np.pi * frequency * t)

    try:
        with sd.OutputStream(samplerate=samplerate, channels=1):
            sd.play(tone, samplerate=samplerate)
            sd.wait()
        print("Test tone played successfully.")
    except Exception as e:
        print(f"Failed to play test tone: {e}")


if __name__ == "__main__":
    log_audio_devices()
    play_test_tone()



__all__ = [
    "ExternalConditionStoppingCriteria",
    "HtmlFileReader",
    "RAGMixin",
    "WeatherMixin",
]


def __getattr__(name):
    if name == "ExternalConditionStoppingCriteria":
        from .external_condition_stopping_criteria import (
            ExternalConditionStoppingCriteria,
        )

        return ExternalConditionStoppingCriteria
    elif name == "HtmlFileReader":
        from .html_file_reader import HtmlFileReader

        return HtmlFileReader
    elif name == "RAGMixin":
        from .rag_mixin import RAGMixin

        return RAGMixin
    elif name == "WeatherMixin":
        from .weather_mixin import WeatherMixin

        return WeatherMixin
    raise AttributeError(f"module {__name__} has no attribute {name}")




from airunner.data.models import ControlnetModel
from airunner.enums import SignalCode
from airunner.gui.widgets.base_widget import BaseWidget
from airunner.gui.widgets.controlnet.templates.controlnet_settings_widget_ui import Ui_controlnet_settings_widget


class ControlnetSettingsWidget(BaseWidget):
    widget_class_ = Ui_controlnet_settings_widget

    def __init__(self, *args, **kwargs):
        self.signal_handlers = {
            SignalCode.APPLICATION_SETTINGS_CHANGED_SIGNAL: self.on_application_settings_changed_signal
        }
        super().__init__(*args, **kwargs)
        self._version = None
        self._load_controlnet_models()

    def controlnet_changed(self, val):
        self.update_controlnet_settings("controlnet", val)

    def on_application_settings_changed_signal(self):
        self._load_controlnet_models()

    def _load_controlnet_models(self):
        if self._version is None or self._version != self.generator_settings.version:
            self._version = self.generator_settings.version
            current_index = 0
            
            controlnet_models = ControlnetModel.objects.filter_by(
                version=self.generator_settings.version
            )
            self.ui.controlnet.blockSignals(True)
            self.ui.controlnet.clear()
            for index, item in enumerate(controlnet_models):
                self.ui.controlnet.addItem(item.display_name)
                if self.controlnet_settings.controlnet == item.display_name:
                    current_index = index
            self.ui.controlnet.setCurrentIndex(current_index)
            self.ui.controlnet.blockSignals(False)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'mistral_license.ui'
##
## Created by: Qt User Interface Compiler version 6.7.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QGridLayout, QLabel, QScrollArea,
    QSizePolicy, QTextBrowser, QWidget)

class Ui_mistral_license(object):
    def setupUi(self, mistral_license):
        if not mistral_license.objectName():
            mistral_license.setObjectName(u"mistral_license")
        mistral_license.resize(746, 1131)
        self.gridLayout = QGridLayout(mistral_license)
        self.gridLayout.setObjectName(u"gridLayout")
        self.scrollArea = QScrollArea(mistral_license)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 726, 1088))
        self.gridLayout_2 = QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.textBrowser = QTextBrowser(self.scrollAreaWidgetContents)
        self.textBrowser.setObjectName(u"textBrowser")

        self.gridLayout_2.addWidget(self.textBrowser, 0, 0, 1, 1)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.gridLayout.addWidget(self.scrollArea, 1, 0, 1, 1)

        self.label = QLabel(mistral_license)
        self.label.setObjectName(u"label")
        font = QFont()
        font.setPointSize(11)
        font.setBold(True)
        self.label.setFont(font)

        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)


        self.retranslateUi(mistral_license)

        QMetaObject.connectSlotsByName(mistral_license)
    # setupUi

    def retranslateUi(self, mistral_license):
        mistral_license.setWindowTitle(QCoreApplication.translate("mistral_license", u"Form", None))
        self.textBrowser.setHtml(QCoreApplication.translate("mistral_license", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><meta charset=\"utf-8\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"hr { height: 1px; border-width: 0; }\n"
"li.unchecked::marker { content: \"\\2610\"; }\n"
"li.checked::marker { content: \"\\2612\"; }\n"
"</style></head><body style=\" font-family:'Ubuntu'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<h1 style=\" margin-top:18px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:xx-large; font-weight:700;\">Mistral AI Research License</span></h1>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">If You want to use a Mistral Model, a Derivative or an Output for any purpose that is not expressly authorized under this Agreement, You must request a license from Mistral AI, whic"
                        "h Mistral AI may grant to You in Mistral AI's sole discretion. To discuss such a license, please contact Mistral AI via the website contact form: <a href=\"https://mistral.ai/contact/\"><span style=\" text-decoration: underline; color:#308cc6;\">https://mistral.ai/contact/</span></a></p>\n"
"<h2 style=\" margin-top:16px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:x-large; font-weight:700;\">1. Scope and acceptance</span></h2>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:700;\">1.1. Scope of the Agreement.</span> This Agreement applies to any use, modification, or Distribution of any Mistral Model by You, regardless of the source You obtained a copy of such Mistral Model.<br /><span style=\" font-weight:700;\">1.2. Acceptance.</span> By accessing, using, modifying, Distributing a Mistral Model, or by creating, using or distributi"
                        "ng a Derivative of the Mistral Model, You agree to be bound by this Agreement.<br /><span style=\" font-weight:700;\">1.3. Acceptance on behalf of a third-party.</span> If You accept this Agreement on behalf of Your employer or another person or entity, You warrant and represent that You have the authority to act and accept this Agreement on their behalf. In such a case, the word &quot;You&quot; in this Agreement will refer to Your employer or such other person or entity.</p>\n"
"<h2 style=\" margin-top:16px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:x-large; font-weight:700;\">2. License</span></h2>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:700;\">2.1. Grant of rights</span>. Subject to Section 3 below, Mistral AI hereby grants You a non-exclusive, royalty-free, worldwide, non-sublicensable, non-transferable, limited licens"
                        "e to use, copy, modify, and Distribute under the conditions provided in Section 2.2 below, the Mistral Model and any Derivatives made by or for Mistral AI and to create Derivatives of the Mistral Model.<br /><span style=\" font-weight:700;\">2.2. Distribution of Mistral Model and Derivatives made by or for Mistral AI.</span> Subject to Section 3 below, You may Distribute copies of the Mistral Model and/or Derivatives made by or for Mistral AI, under the following conditions: You must make available a copy of this Agreement to third-party recipients of the Mistral Models and/or Derivatives made by or for Mistral AI you Distribute, it being specified that any rights to use the Mistral Models and/or Derivatives made by or for Mistral AI shall be directly granted by Mistral AI to said third-party recipients pursuant to the Mistral AI Research License agreement executed between these parties; You must retain in all copies of the Mistral Models the following attribution notice within a &quot;Notice&quot; text file d"
                        "istributed as part of such copies: &quot;Licensed by Mistral AI under the Mistral AI Research License&quot;.<br /><span style=\" font-weight:700;\">2.3. Distribution of Derivatives made by or for You.</span> Subject to Section 3 below, You may Distribute any Derivatives made by or for You under additional or different terms and conditions, provided that: In any event, the use and modification of Mistral Model and/or Derivatives made by or for Mistral AI shall remain governed by the terms and conditions of this Agreement; You include in any such Derivatives made by or for You prominent notices stating that You modified the concerned Mistral Model; and Any terms and conditions You impose on any third-party recipients relating to Derivatives made by or for You shall neither limit such third-party recipients' use of the Mistral Model or any Derivatives made by or for Mistral AI in accordance with the Mistral AI Research License nor conflict with any of its terms and conditions.</p>\n"
"<h2 style=\" margin-top:16px"
                        "; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:x-large; font-weight:700;\">3. Limitations</span></h2>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:700;\">3.1. Misrepresentation.</span> You must not misrepresent or imply, through any means, that the Derivatives made by or for You and/or any modified version of the Mistral Model You Distribute under your name and responsibility is an official product of Mistral AI or has been endorsed, approved or validated by Mistral AI, unless You are authorized by Us to do so in writing.<br /><span style=\" font-weight:700;\">3.2. Usage Limitation.</span> You shall only use the Mistral Models, Derivatives (whether or not created by Mistral AI) and Outputs for Research Purposes.</p>\n"
"<h2 style=\" margin-top:16px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-in"
                        "dent:0px;\"><span style=\" font-size:x-large; font-weight:700;\">4. Intellectual Property</span></h2>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:700;\">4.1. Trademarks.</span> No trademark licenses are granted under this Agreement, and in connection with the Mistral Models, You may not use any name or mark owned by or associated with Mistral AI or any of its affiliates, except (i) as required for reasonable and customary use in describing and Distributing the Mistral Models and Derivatives made by or for Mistral AI and (ii) for attribution purposes as required by this Agreement.<br /><span style=\" font-weight:700;\">4.2. Outputs.</span> We claim no ownership rights in and to the Outputs. You are solely responsible for the Outputs You generate and their subsequent uses in accordance with this Agreement. Any Outputs shall be subject to the restrictions set out in Section 3 of this Agreement.<br /><span"
                        " style=\" font-weight:700;\">4.3. Derivatives.</span> By entering into this Agreement, You accept that any Derivatives that You may create or that may be created for You shall be subject to the restrictions set out in Section 3 of this Agreement.</p>\n"
"<h2 style=\" margin-top:16px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:x-large; font-weight:700;\">5. Liability</span></h2>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:700;\">5.1. Limitation of liability.</span> In no event, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall Mistral AI be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this Agreement or out of the use or inability to use the Mistral Models and Derivativ"
                        "es (including but not limited to damages for loss of data, loss of goodwill, loss of expected profit or savings, work stoppage, computer failure or malfunction, or any damage caused by malware or security breaches), even if Mistral AI has been advised of the possibility of such damages.<br /><span style=\" font-weight:700;\">5.2. Indemnification.</span> You agree to indemnify and hold harmless Mistral AI from and against any claims, damages, or losses arising out of or related to Your use or Distribution of the Mistral Models and Derivatives.</p>\n"
"<h2 style=\" margin-top:16px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:x-large; font-weight:700;\">6. Warranty</span></h2>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:700;\">6.1. Disclaimer.</span> Unless required by applicable law or prior agreed to by Mistral AI in writing, Mis"
                        "tral AI provides the Mistral Models and Derivatives on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. Mistral AI does not represent nor warrant that the Mistral Models and Derivatives will be error-free, meet Your or any third party's requirements, be secure or will allow You or any third party to achieve any kind of result or generate any kind of content. You are solely responsible for determining the appropriateness of using or Distributing the Mistral Models and Derivatives and assume any risks associated with Your exercise of rights under this Agreement.</p>\n"
"<h2 style=\" margin-top:16px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:x-large; font-weight:700;\">7. Termination</span></h2>\n"
"<p style=\" margin-top:12px; margin-bottom:12px;"
                        " margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:700;\">7.1. Term.</span> This Agreement is effective as of the date of your acceptance of this Agreement or access to the concerned Mistral Models or Derivatives and will continue until terminated in accordance with the following terms.<br /><span style=\" font-weight:700;\">7.2. Termination.</span> Mistral AI may terminate this Agreement at any time if You are in breach of this Agreement. Upon termination of this Agreement, You must cease to use all Mistral Models and Derivatives and shall permanently delete any copy thereof. The following provisions, in their relevant parts, will survive any termination or expiration of this Agreement, each for the duration necessary to achieve its own intended purpose (e.g. the liability provision will survive until the end of the applicable limitation period):Sections 5 (Liability), 6(Warranty), 7 (Termination) and 8 (General Provisions).<br /><span style=\" font-weight:70"
                        "0;\">7.3. Litigation.</span> If You initiate any legal action or proceedings against Us or any other entity (including a cross-claim or counterclaim in a lawsuit), alleging that the Model or a Derivative, or any part thereof, infringe upon intellectual property or other rights owned or licensable by You, then any licenses granted to You under this Agreement will immediately terminate as of the date such legal action or claim is filed or initiated.</p>\n"
"<h2 style=\" margin-top:16px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:x-large; font-weight:700;\">8. General provisions</span></h2>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:700;\">8.1. Governing laws.</span> This Agreement will be governed by the laws of France, without regard to choice of law principles, and the UN Convention on Contracts for the International Sale of G"
                        "oods does not apply to this Agreement.<br /><span style=\" font-weight:700;\">8.2. Competent jurisdiction.</span> The courts of Paris shall have exclusive jurisdiction of any dispute arising out of this Agreement.<br /><span style=\" font-weight:700;\">8.3. Severability.</span> If any provision of this Agreement is held to be invalid, illegal or unenforceable, the remaining provisions shall be unaffected thereby and remain valid as if such provision had not been set forth herein.</p>\n"
"<h2 style=\" margin-top:16px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:x-large; font-weight:700;\">9. Definitions</span></h2>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">&quot;Agreement&quot;: means this Mistral AI Research License agreement governing the access, use, and Distribution of the Mistral Models, Derivatives and Outputs.<br />&quot;Derivative&quot;: means an"
                        "y (i) modified version of the Mistral Model (including but not limited to any customized or fine-tuned version thereof), (ii) work based on the Mistral Model, or (iii) any other derivative work thereof.<br />&quot;Distribution&quot;, &quot;Distributing&quot;, &quot;Distribute&quot; or &quot;Distributed&quot;: means supplying, providing or making available, by any means, a copy of the Mistral Models and/or the Derivatives as the case may be, subject to Section 3 of this Agreement.<br />&quot;Mistral AI&quot;, &quot;We&quot; or &quot;Us&quot;: means Mistral AI, a French soci\u00e9t\u00e9 par actions simplifi\u00e9e registered in the Paris commercial registry under the number 952 418 325, and having its registered seat at 15, rue des Halles, 75001 Paris.<br />&quot;Mistral Model&quot;: means the foundational large language model(s), and its elements which include algorithms, software, instructed checkpoints, parameters, source code (inference code, evaluation code and, if applicable, fine-tuning code) and any oth"
                        "er elements associated thereto made available by Mistral AI under this Agreement, including, if any, the technical documentation, manuals and instructions for the use and operation thereof.<br />&quot;Research Purposes&quot;: means any use of a Mistral Model, Derivative, or Output that is solely for (a) personal, scientific or academic research, and (b) for non-profit and non-commercial purposes, and not directly or indirectly connected to any commercial activities or business operations. For illustration purposes, Research Purposes does not include (1) any usage of the Mistral Model, Derivative or Output by individuals or contractors employed in or engaged by companies in the context of (a) their daily tasks, or (b) any activity (including but not limited to any testing or proof-of-concept) that is intended to generate revenue, nor (2) any Distribution by a commercial entity of the Mistral Model, Derivative or Output whether in return for payment or free of charge, in any medium or form, including but not lim"
                        "ited to through a hosted or managed service (e.g. SaaS, cloud instances, etc.), or behind a software layer.<br />&quot;Outputs&quot;: means any content generated by the operation of the Mistral Models or the Derivatives from a prompt (i.e., text instructions) provided by users. For the avoidance of doubt, Outputs do not include any components of a Mistral Models, such as any fine-tuned versions of the Mistral Models, the weights, or parameters.<br />&quot;You&quot;: means the individual or entity entering into this Agreement with Mistral AI.</p></body></html>", None))
        self.label.setText(QCoreApplication.translate("mistral_license", u"Mistral AI Research License", None))
    # retranslateUi

